"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
var AnnotationBuilder_1;
const core_1 = require("@ts-ioc/core");
const IAnnotationBuilder_1 = require("./IAnnotationBuilder");
const runnable_1 = require("../runnable");
const AnnoBuildStrategy_1 = require("./AnnoBuildStrategy");
/**
 * Annotation class builder. build class with metadata and config.
 *
 * @export
 * @class AnnotationBuilder
 * @implements {implements IAnnotationBuilder<T>}
 * @template T
 */
let AnnotationBuilder = AnnotationBuilder_1 = class AnnotationBuilder {
    constructor() {
    }
    /**
     * get metadata accessor.
     *
     * @param {(Token<any>|| AnnotationConfigure<T>)} token
     * @param {AnnotationConfigure<T>} [config]
     * @returns {IMetaAccessor<any>}
     * @memberof AnnotationBuilder
     */
    getMetaAccessor(token, config) {
        let mtk;
        if (core_1.isToken(token)) {
            mtk = token;
        }
        else {
            config = token;
        }
        return this.container.getService(core_1.MetaAccessorToken, mtk ? [mtk, core_1.lang.getClass(this)] : core_1.lang.getClass(this), tk => new core_1.InjectMetaAccessorToken(tk), config ? (config.defaultMetaAccessor || core_1.MetaAccessorToken) : core_1.MetaAccessorToken);
    }
    /**
     * build token type with config.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} token
     * @param {(AnnotationConfigure<T> | BuildOptions<T>)} [config]
     * @param {BuildOptions<T>} [options]
     * @returns {Promise<T>}
     * @memberof AnnotationBuilder
     */
    build(token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let tk;
            let cfg;
            if (core_1.isToken(token)) {
                tk = token;
                if (core_1.isUndefined(options)) {
                    options = config;
                    cfg = null;
                }
                else {
                    cfg = config;
                }
                cfg = this.getMetaAccessor(tk, cfg).getMetadata(tk, this.container, cfg);
            }
            else {
                options = config;
                cfg = token;
                let accessor = this.getMetaAccessor(cfg);
                tk = accessor.getToken(cfg, this.container);
                if (core_1.isToken(tk)) {
                    cfg = accessor.getMetadata(tk, this.container, cfg);
                }
            }
            if (core_1.isClass(tk) && !this.container.hasRegister(tk)) {
                this.container.register(tk);
            }
            if (options && core_1.isFunction(options.vaild) && !options.vaild(tk)) {
                return null;
            }
            let builder = (options && options.builder) ? options.builder : this.getBuilder(tk, cfg);
            if (!this.isEqual(builder)) {
                return yield builder.build(tk, cfg, core_1.lang.assign(options || {}, { builder: builder }));
            }
            else {
                yield this.registerExts(cfg);
                let instance = yield this.createInstance(tk, cfg, options);
                if (!instance) {
                    return null;
                }
                let strategy = this.container.getService(AnnoBuildStrategy_1.AnnoBuildStrategyToken, [
                    tk || core_1.lang.getClass(instance),
                    ...((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []),
                    { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
                ], tk => new AnnoBuildStrategy_1.InjectAnnoBuildStrategyToken(tk));
                if (strategy) {
                    yield strategy.build(instance, cfg, options);
                }
                if (options && core_1.isFunction(options.onCompleted)) {
                    options.onCompleted(tk, cfg, instance, this);
                }
                return instance;
            }
        });
    }
    /**
     * run runable.
     *
     * @param {(Token<T> | AnnotationConfigure<T>)} runable
     * @param {*} [data] bootstrap data, build data, Runnable data.
     * @returns {Promise<Runnable<T>>}
     * @memberof IGModuleBuilder
     */
    boot(runable, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let instance;
            let builder;
            let tk;
            let cfg;
            let onCompleted = (token, config, hook, builder) => {
                tk = token;
                cfg = config;
                instance = hook;
                builder = builder;
            };
            if (core_1.isToken(runable)) {
                if (core_1.isUndefined(options)) {
                    options = config;
                    cfg = null;
                }
                else {
                    cfg = config;
                }
                yield this.build(runable, cfg, core_1.lang.assign({
                    onCompleted: onCompleted
                }, options));
            }
            else {
                yield this.build(runable, core_1.lang.assign({
                    onCompleted: onCompleted
                }, config));
            }
            builder = builder || this;
            let runner = builder.resolveRunable(instance, cfg, tk, options);
            if (runner && core_1.isFunction(runner.onInit)) {
                yield runner.onInit(options);
            }
            let data = options ? options.data : undefined;
            if (runnable_1.isRunner(runner)) {
                yield runner.run(data);
            }
            else if (runnable_1.isService(runner)) {
                yield runner.start(data);
            }
            return runner;
        });
    }
    /**
     * run annotation instance.
     *
     * @param {T} instance
     * @param {AnnotationConfigure<T>} [config]
     * @param {Token<T>} [token]
     * @returns {Promise<Runnable<T>>}
     * @memberof AnnotationBuilder
     */
    resolveRunable(instance, config, token, options) {
        if (!instance) {
            return null;
        }
        let tk;
        if (core_1.isToken(token)) {
            tk = token;
        }
        else {
            options = token;
        }
        tk = tk || core_1.lang.getClass(instance);
        if (!config) {
            config = this.getMetaAccessor(tk).getMetadata(tk, this.container);
        }
        if (instance instanceof runnable_1.Runner || instance instanceof runnable_1.Service) {
            return instance;
        }
        else {
            let providers = [{ provide: tk, useValue: instance }, { token: tk, instance: instance, config: config }];
            return this.container.getService([runnable_1.RunnerToken, runnable_1.ServiceToken], [
                tk,
                ...((options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []),
                { target: core_1.lang.getClass(this), level: core_1.RefTagLevel.self }
            ], tk => new runnable_1.InjectRunnableToken(tk), config.defaultRunnable || true, ...providers);
        }
    }
    createInstance(token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!core_1.isToken(token)) {
                return null;
            }
            let instance = this.resolveToken(token, options ? options.target : undefined);
            if (core_1.isNullOrUndefined(instance)) {
                console.log(`can not find token ${token ? token.toString() : null} in container.`);
                return null;
            }
            return instance;
        });
    }
    getBuilder(token, config, options) {
        let builder;
        if (config && config.annoBuilder) {
            if (core_1.isClass(config.annoBuilder)) {
                if (!this.container.has(config.annoBuilder)) {
                    this.container.register(config.annoBuilder);
                }
            }
            if (core_1.isToken(config.annoBuilder)) {
                builder = this.container.resolve(config.annoBuilder);
            }
            else if (config.annoBuilder instanceof AnnotationBuilder_1) {
                builder = config.annoBuilder;
            }
        }
        if (options && options.builder) {
            builder = options.builder;
        }
        if (!builder && token) {
            builder = this.container.getService(IAnnotationBuilder_1.AnnotationBuilderToken, [
                token,
                ...(options && options.target) ? [{ target: core_1.lang.getClass(options.target), level: core_1.RefTagLevel.self }] : []
            ], (tk) => new IAnnotationBuilder_1.InjectAnnotationBuilder(tk), config.defaultAnnoBuilder || IAnnotationBuilder_1.AnnotationBuilderToken);
        }
        if (builder && !builder.container) {
            builder.container = this.container;
        }
        return builder || this;
    }
    /**
     * register extension before create instance.
     *
     * @protected
     * @param {AnnotationConfigure<T>} [config]
     * @memberof AnnotationBuilder
     */
    registerExts(config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
        });
    }
    isEqual(build) {
        if (!build) {
            return false;
        }
        if (build === this) {
            return true;
        }
        if (core_1.lang.getClass(build) === core_1.lang.getClass(this)) {
            return true;
        }
        return false;
    }
    resolveToken(token, target) {
        if (target) {
            return this.container.getService(token, target);
        }
        return this.container.resolve(token);
    }
};
AnnotationBuilder.classAnnations = { "name": "AnnotationBuilder", "params": { "constructor": [], "getMetaAccessor": ["token", "config"], "build": ["token", "config", "options"], "boot": ["runable", "config", "options"], "resolveRunable": ["instance", "config", "token", "options"], "createInstance": ["token", "config", "options"], "getBuilder": ["token", "config", "options"], "registerExts": ["config"], "isEqual": ["build"], "resolveToken": ["token", "target"] } };
tslib_1.__decorate([
    core_1.Inject(core_1.ContainerToken),
    tslib_1.__metadata("design:type", Object)
], AnnotationBuilder.prototype, "container", void 0);
AnnotationBuilder = AnnotationBuilder_1 = tslib_1.__decorate([
    core_1.Singleton(IAnnotationBuilder_1.AnnotationBuilderToken),
    tslib_1.__metadata("design:paramtypes", [])
], AnnotationBuilder);
exports.AnnotationBuilder = AnnotationBuilder;

//# sourceMappingURL=../sourcemaps/annotations/AnnotationBuilder.js.map
