"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@ts-ioc/core");
const IConfigureManager_1 = require("./IConfigureManager");
/**
 * configure manager.
 *
 * @export
 * @class ConfigureManager
 */
let ConfigureManager = class ConfigureManager {
    /**
     * Creates an instance of ConfigureManager.
     * @param {string} [baseURL]
     * @memberof ConfigureManager
     */
    constructor(baseURL) {
        this.baseURL = baseURL;
        this.configs = [];
    }
    /**
     * use configuration.
     *
     * @param {(string | AppConfigure)} [config]
     * @returns {this} this configure manager.
     * @memberof Bootstrap
     */
    useConfiguration(config) {
        if (core_1.isUndefined(config)) {
            config = '';
        }
        // clean cached config.
        this.config = null;
        let idx = this.configs.indexOf(config);
        if (idx >= 0) {
            this.configs.splice(idx, 1);
        }
        this.configs.push(config);
        return this;
    }
    /**
     * get config.
     *
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    getConfig() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.config) {
                this.config = yield this.initConfig();
            }
            return this.config;
        });
    }
    /**
     * init config.
     *
     * @protected
     * @returns
     * @memberof ConfigureManager
     */
    initConfig() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let config = yield this.getDefaultConfig();
            if (this.configs.length < 1) {
                this.configs.push(''); // load default loader config.
            }
            let exts = yield Promise.all(this.configs.map(cfg => {
                if (core_1.isString(cfg)) {
                    return this.loadConfig(cfg);
                }
                else {
                    return cfg;
                }
            }));
            exts.forEach(exCfg => {
                if (exCfg) {
                    core_1.lang.assign(config, exCfg);
                }
            });
            return config;
        });
    }
    /**
     * load config.
     *
     * @protected
     * @param {string} src
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    loadConfig(src) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.container.has(IConfigureManager_1.ConfigureLoaderToken)) {
                let loader = this.container.resolve(IConfigureManager_1.ConfigureLoaderToken, { baseURL: this.baseURL, container: this.container });
                return yield loader.load(src);
            }
            else if (src) {
                let builder = this.container.getBuilder();
                let cfg = yield builder.loader.load([src]);
                return cfg.length ? cfg[0] : null;
            }
            else {
                return null;
            }
        });
    }
    /**
     * get default config.
     *
     * @protected
     * @returns {Promise<T>}
     * @memberof ConfigureManager
     */
    getDefaultConfig() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.container.has(IConfigureManager_1.DefaultConfigureToken)) {
                return this.container.resolve(IConfigureManager_1.DefaultConfigureToken);
            }
            else {
                return {};
            }
        });
    }
};
ConfigureManager.classAnnations = { "name": "ConfigureManager", "params": { "constructor": ["baseURL"], "useConfiguration": ["config"], "getConfig": [], "initConfig": [], "loadConfig": ["src"], "getDefaultConfig": [] } };
tslib_1.__decorate([
    core_1.Inject(core_1.ContainerToken),
    tslib_1.__metadata("design:type", Object)
], ConfigureManager.prototype, "container", void 0);
ConfigureManager = tslib_1.__decorate([
    core_1.Singleton(IConfigureManager_1.ConfigureMgrToken),
    tslib_1.__metadata("design:paramtypes", [String])
], ConfigureManager);
exports.ConfigureManager = ConfigureManager;

//# sourceMappingURL=../sourcemaps/boot/ConfigureManager.js.map
