import { IContainer, LoadType, Factory, Token, IContainerBuilder, ParamProviders } from '@ts-ioc/core';
import { IRunnableBuilder, CustomRegister, RunOptions } from './IRunnableBuilder';
import { ModuleBuilder, InjectedModule, IModuleBuilder, ModuleConfig } from '../modules';
import { ContainerPool, Events, IEvents } from '../utils';
import { Runnable } from '../runnable';
import { IConfigureManager } from './IConfigureManager';
import { RunnableConfigure } from './AppConfigure';
/**
 * runnable events
 *
 * @export
 * @enum {number}
 */
export declare enum RunnableEvents {
    /**
     * on root container created.
     */
    onRootContainerCreated = "onRootContainerCreated",
    /**
     * on root container inited.
     */
    onRootContainerInited = "onRootContainerInited",
    /**
     * on module created.
     */
    onModuleCreated = "onModuleCreated",
    /**
     * on registered runner use module.
     */
    registeredExt = "registeredExt"
}
/**
 * runnable builder.
 *
 * @export
 * @class RunnableBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
export declare class RunnableBuilder<T> extends ModuleBuilder<T> implements IRunnableBuilder<T>, IEvents {
    protected globalModules: LoadType[];
    protected customRegs: CustomRegister<T>[];
    protected beforeInitPds: Map<Token<any>, any>;
    protected afterInitPds: Map<Token<any>, any>;
    protected configMgr: IConfigureManager<ModuleConfig<T>>;
    inited: boolean;
    events: Events;
    private _baseURL;
    constructor(baseURL?: string);
    protected initEvents(): void;
    on(name: string, event: (...args: any[]) => void): this;
    off(name: string, event?: (...args: any[]) => void): this;
    emit(name: string, ...args: any[]): void;
    getPools(): ContainerPool;
    getRunRoot(container: IContainer): string;
    /**
     * has register in pools.
     * use must after `initContainerPools`.
     *
     * @template T
     * @param {Token<T>} key
     * @returns {boolean}
     * @memberof RunnableBuilder
     */
    hasRegister<T>(key: Token<T>): boolean;
    /**
     * resove token in pools.
     * use must after `initContainerPools`.
     * @template T
     * @param {Token<T>} token
     * @param {...ParamProviders[]} providers
     * @returns {T}
     * @memberof RunnableBuilder
     */
    resolve<T>(token: Token<T>, ...providers: ParamProviders[]): T;
    protected createContainerBuilder(): IContainerBuilder;
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof RunnableBuilder
     */
    use(...modules: LoadType[]): this;
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforeInit]
     * @returns {this}
     * @memberof RunnableBuilder
     */
    provider(provide: Token<any>, provider: Token<any> | Factory<any>, beforeInit?: boolean): this;
    getProvider(provide: Token<any>, beforeInit?: boolean): Token<any> | Factory<any>;
    initContainerPools(): Promise<void>;
    load(token: Token<T> | RunnableConfigure, config?: RunnableConfigure | RunOptions<T>, options?: RunOptions<T>): Promise<InjectedModule<T>>;
    bootstrap(token: Token<T> | RunnableConfigure, config?: RunnableConfigure | RunOptions<T>, options?: RunOptions<T>): Promise<Runnable<T>>;
    /**
     * get module builder
     *
     * @param {(Token<T> |RunnableConfigure)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof RunnableBuilder
     */
    getBuilderByConfig(token: Token<T> | RunnableConfigure, options?: RunOptions<T>): Promise<IModuleBuilder<T>>;
    getBuilder(injmdl: InjectedModule<T>): IModuleBuilder<T>;
    getConfigManager(): IConfigureManager<ModuleConfig<T>>;
    protected createConfigureMgr(): IConfigureManager<RunnableConfigure>;
    protected createDefaultContainer(): IContainer;
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @memberof RunnableBuilder
     */
    protected registerExts(container: IContainer): Promise<void>;
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    protected registerByConfigure(container: IContainer, config: RunnableConfigure): Promise<void>;
    static classAnnations: any;
}
