"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@ts-ioc/core");
const IRunnableBuilder_1 = require("./IRunnableBuilder");
const modules_1 = require("../modules");
const utils_1 = require("../utils");
const BootModule_1 = require("../BootModule");
const IConfigureManager_1 = require("./IConfigureManager");
const ConfigureRegister_1 = require("./ConfigureRegister");
/**
 * runnable events
 *
 * @export
 * @enum {number}
 */
var RunnableEvents;
(function (RunnableEvents) {
    /**
     * on root container created.
     */
    RunnableEvents["onRootContainerCreated"] = "onRootContainerCreated";
    /**
     * on root container inited.
     */
    RunnableEvents["onRootContainerInited"] = "onRootContainerInited";
    /**
     * on module created.
     */
    RunnableEvents["onModuleCreated"] = "onModuleCreated";
    /**
     * on registered runner use module.
     */
    RunnableEvents["registeredExt"] = "registeredExt";
})(RunnableEvents = exports.RunnableEvents || (exports.RunnableEvents = {}));
/**
 * runnable builder.
 *
 * @export
 * @class RunnableBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
let RunnableBuilder = class RunnableBuilder extends modules_1.ModuleBuilder {
    constructor(baseURL) {
        super();
        this.inited = false;
        this._baseURL = baseURL;
        this.customRegs = [];
        this.globalModules = [];
        this.beforeInitPds = new Map();
        this.afterInitPds = new Map();
        this.events = new utils_1.Events();
        this.initEvents();
    }
    initEvents() {
        this.on(RunnableEvents.onRootContainerInited, (container) => {
            this.afterInitPds.forEach((val, key) => {
                container.bindProvider(key, val);
            });
        });
    }
    on(name, event) {
        this.events.on(name, event);
        return this;
    }
    off(name, event) {
        this.events.off(name, event);
        return this;
    }
    emit(name, ...args) {
        this.events.emit(name, ...args);
    }
    getPools() {
        if (!this.pools) {
            this.pools = new utils_1.ContainerPool(this.createContainerBuilder());
            this.createDefaultContainer();
        }
        return this.pools;
    }
    getRunRoot(container) {
        return this._baseURL || container.get(IRunnableBuilder_1.ProcessRunRootToken) || '';
    }
    /**
     * has register in pools.
     * use must after `initContainerPools`.
     *
     * @template T
     * @param {Token<T>} key
     * @returns {boolean}
     * @memberof RunnableBuilder
     */
    hasRegister(key) {
        return this.getPools().values().some(c => c.hasRegister(key));
    }
    /**
     * resove token in pools.
     * use must after `initContainerPools`.
     * @template T
     * @param {Token<T>} token
     * @param {...ParamProviders[]} providers
     * @returns {T}
     * @memberof RunnableBuilder
     */
    resolve(token, ...providers) {
        let resolved;
        this.getPools().values().some(c => {
            resolved = c.resolve(token, ...providers);
            return !core_1.isNullOrUndefined(resolved);
        });
        return resolved;
    }
    createContainerBuilder() {
        return new core_1.ContainerBuilder();
    }
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof RunnableBuilder
     */
    use(...modules) {
        this.globalModules = this.globalModules.concat(modules);
        this.inited = false;
        return this;
    }
    /**
     * bind provider
     *
     * @template T
     * @param {Token<T>} provide
     * @param {Token<T> | Factory<T>} provider
     * @param {boolean} [beforeInit]
     * @returns {this}
     * @memberof RunnableBuilder
     */
    provider(provide, provider, beforeInit) {
        if (beforeInit) {
            this.beforeInitPds.set(provide, provider);
        }
        else {
            this.afterInitPds.set(provide, provider);
        }
        return this;
    }
    getProvider(provide, beforeInit) {
        if (beforeInit) {
            return this.beforeInitPds.has(provide) ? this.beforeInitPds.get(provide) : null;
        }
        else {
            return this.afterInitPds.has(provide) ? this.afterInitPds.get(provide) : null;
        }
    }
    initContainerPools() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.inited) {
                return;
            }
            let container = this.getPools().getDefault();
            yield this.registerExts(container);
            let configManager = this.getConfigManager();
            let config = yield configManager.getConfig();
            yield this.registerByConfigure(container, config);
            this.inited = true;
            this.events.emit(RunnableEvents.onRootContainerInited, container);
        });
    }
    load(token, config, options) {
        const _super = Object.create(null, {
            load: { get: () => super.load }
        });
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.initContainerPools();
            return yield _super.load.call(this, token, config, options);
        });
    }
    bootstrap(token, config, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let params = this.vaildParams(token, config, options);
            options = params.options || {};
            let injmdl = params.token ? yield this.load(params.token, params.config, options) : yield this.load(params.config, options);
            options.env = injmdl;
            let builder = this.getBuilder(injmdl);
            options.bootBuilder = this;
            options.configManager = this.getConfigManager();
            return params.token ? yield builder.bootstrap(params.token, params.config, options) : yield builder.bootstrap(params.config, options);
        });
    }
    /**
     * get module builder
     *
     * @param {(Token<T> |RunnableConfigure)} token
     * @param {ModuleEnv} [env]
     * @returns {IModuleBuilder<T>}
     * @memberof RunnableBuilder
     */
    getBuilderByConfig(token, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let injmdl = yield this.load(token, options);
            return this.getBuilder(injmdl);
        });
    }
    getBuilder(injmdl) {
        let cfg = injmdl.config;
        let container = injmdl.container;
        let builder;
        if (cfg) {
            if (core_1.isClass(cfg.builder)) {
                if (!container.has(cfg.builder)) {
                    container.register(cfg.builder);
                }
            }
            if (core_1.isToken(cfg.builder)) {
                builder = container.resolve(cfg.builder);
            }
            else if (cfg.builder instanceof modules_1.ModuleBuilder) {
                builder = cfg.builder;
            }
        }
        let tko = injmdl.token;
        if (!builder && tko) {
            builder = container.getService(modules_1.ModuleBuilderToken, tko, (tk) => new modules_1.InjectModuleBuilderToken(tk), cfg.defaultBuilder || modules_1.ModuleBuilderToken);
        }
        return builder || this;
    }
    getConfigManager() {
        if (!this.configMgr) {
            this.configMgr = this.createConfigureMgr();
        }
        return this.configMgr;
    }
    createConfigureMgr() {
        let container = this.getPools().getDefault();
        return container.getService(IConfigureManager_1.ConfigureMgrToken, core_1.lang.getClass(this), { baseURL: this.getRunRoot(container) });
    }
    createDefaultContainer() {
        let container = this.pools.getDefault();
        container.register(BootModule_1.BootModule);
        let chain = container.getBuilder().getInjectorChain(container);
        chain.first(container.resolve(modules_1.DIModuleInjectorToken));
        container.bindProvider(utils_1.ContainerPoolToken, () => this.getPools());
        this.beforeInitPds.forEach((val, key) => {
            container.bindProvider(key, val);
        });
        this.events.emit(RunnableEvents.onRootContainerCreated, container);
        return container;
    }
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfigure} config
     * @memberof RunnableBuilder
     */
    registerExts(container) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.globalModules.length) {
                let usedModules = this.globalModules;
                let types = yield container.loadModule(...usedModules);
                this.emit(RunnableEvents.registeredExt, types, container);
            }
        });
    }
    /**
     * register by configure.
     *
     * @protected
     * @param {IContainer} container
     * @param {ModuleConfig<T>} config
     * @returns {Promise<void>}
     * @memberof RunnableBuilder
     */
    registerByConfigure(container, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield core_1.PromiseUtil.step(this.customRegs.map(cs => () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let tokens = yield cs(container, config, this);
                return tokens;
            })));
            let curClass = core_1.lang.getClass(this);
            let regs = [];
            this.getPools().iterator(c => {
                if (!config.baseURL) {
                    config.baseURL = this.getRunRoot(c);
                }
                let registers = c.getServices(ConfigureRegister_1.ConfigureRegister, curClass, true, core_1.ResoveWay.current);
                if (registers && registers.length) {
                    regs.push({
                        container: c,
                        registers: registers
                    });
                }
            });
            yield Promise.all(regs.map(reg => {
                return Promise.all(reg.registers.map(r => r.register(config, reg.container, this)));
            }));
        });
    }
};
RunnableBuilder.classAnnations = { "name": "RunnableBuilder", "params": { "constructor": ["baseURL"], "initEvents": [], "on": ["name", "event"], "off": ["name", "event"], "emit": ["name", "args"], "getPools": [], "getRunRoot": ["container"], "hasRegister": ["key"], "resolve": ["token", "providers"], "createContainerBuilder": [], "use": ["modules"], "provider": ["provide", "provider", "beforeInit"], "getProvider": ["provide", "beforeInit"], "initContainerPools": [], "load": ["token", "config", "options"], "bootstrap": ["token", "config", "options"], "getBuilderByConfig": ["token", "options"], "getBuilder": ["injmdl"], "getConfigManager": [], "createConfigureMgr": [], "createDefaultContainer": [], "registerExts": ["container"], "registerByConfigure": ["container", "config"] } };
RunnableBuilder = tslib_1.__decorate([
    core_1.Injectable(IRunnableBuilder_1.RunnableBuilderToken),
    tslib_1.__metadata("design:paramtypes", [String])
], RunnableBuilder);
exports.RunnableBuilder = RunnableBuilder;

//# sourceMappingURL=../sourcemaps/boot/RunnableBuilder.js.map
